@@ -62,6 +62,7 @@ import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Set;
+import org.apache.http.HttpEntity;
 import org.apache.http.HttpResponse;
 import org.apache.http.NameValuePair;
 import org.apache.http.StatusLine;
@@ -74,6 +75,7 @@ import org.apache.http.entity.ContentType;
 import org.apache.http.entity.InputStreamEntity;
 import org.apache.http.impl.client.DefaultHttpClient;
 import org.apache.http.message.BasicNameValuePair;
+import org.apache.http.util.EntityUtils;
 import org.json.JSONException;
 import org.json.JSONObject;
 import org.json.JSONValue;
@@ -90,7 +92,7 @@ import com.openexchange.exception.OXException;
 import com.openexchange.file.storage.FileStorageAccount;
 import com.openexchange.file.storage.FileStorageExceptionCodes;
 import com.openexchange.file.storage.FileStorageFolder;
-import com.openexchange.file.storage.onedrive.access.OneDriveAccess;
+import com.openexchange.file.storage.onedrive.access.OneDriveOAuthAccess;
 import com.openexchange.java.Charsets;
 import com.openexchange.java.Streams;
 import com.openexchange.session.Session;
@@ -147,15 +149,16 @@ public abstract class AbstractOneDriveResourceAccess {
          *
          * @param httpResponse The HTTP response
          * @throws OXException If an Open-Xchange error is yielded from status
+         * @throws ClientProtocolException To singal a client error
          */
-        void handleStatusCode(HttpResponse httpResponse) throws OXException;
+        void handleStatusCode(HttpResponse httpResponse) throws OXException, ClientProtocolException;
     }
 
     /** The default status code policy; accepting greater than/equal to <code>200</code> and lower than <code>300</code> */
     public static final StatusCodePolicy STATUS_CODE_POLICY_DEFAULT = new StatusCodePolicy() {
 
         @Override
-        public void handleStatusCode(HttpResponse httpResponse) throws OXException {
+        public void handleStatusCode(HttpResponse httpResponse) throws OXException, ClientProtocolException {
             StatusLine statusLine = httpResponse.getStatusLine();
             int statusCode = statusLine.getStatusCode();
             if (statusCode < 200 || statusCode >= 300) {
@@ -164,8 +167,15 @@ public abstract class AbstractOneDriveResourceAccess {
                 }
                 String reason;
                 try {
-                    JSONObject jsonObject = new JSONObject(new InputStreamReader(httpResponse.getEntity().getContent(), Charsets.UTF_8));
-                    reason = jsonObject.getJSONObject("error").getString("message");
+                    JSONObject jResponse = new JSONObject(new InputStreamReader(httpResponse.getEntity().getContent(), Charsets.UTF_8));
+                    JSONObject jError = jResponse.getJSONObject("error");
+                    String code = jError.getString("code");
+                    if ("resource_already_exists".equals(code)) {
+                        throw new DuplicateResourceException(jError.getString("message"));
+                    }
+                    reason = jError.getString("message");
+                } catch (DuplicateResourceException e) {
+                    throw e;
                 } catch (Exception e) {
                     reason = statusLine.getReasonPhrase();
                 }
@@ -178,14 +188,21 @@ public abstract class AbstractOneDriveResourceAccess {
     public static final StatusCodePolicy STATUS_CODE_POLICY_IGNORE_NOT_FOUND = new StatusCodePolicy() {
 
         @Override
-        public void handleStatusCode(HttpResponse httpResponse) throws OXException {
+        public void handleStatusCode(HttpResponse httpResponse) throws OXException, ClientProtocolException {
             StatusLine statusLine = httpResponse.getStatusLine();
             int statusCode = statusLine.getStatusCode();
             if ((statusCode < 200 || statusCode >= 300) && statusCode != 404) {
                 String reason;
                 try {
-                    JSONObject jsonObject = new JSONObject(new InputStreamReader(httpResponse.getEntity().getContent(), Charsets.UTF_8));
-                    reason = jsonObject.getJSONObject("error").getString("message");
+                    JSONObject jResponse = new JSONObject(new InputStreamReader(httpResponse.getEntity().getContent(), Charsets.UTF_8));
+                    JSONObject jError = jResponse.getJSONObject("error");
+                    String code = jError.getString("code");
+                    if ("resource_already_exists".equals(code)) {
+                        throw new DuplicateResourceException(jError.getString("message"));
+                    }
+                    reason = jError.getString("message");
+                } catch (DuplicateResourceException e) {
+                    throw e;
                 } catch (Exception e) {
                     reason = statusLine.getReasonPhrase();
                 }
@@ -224,7 +241,7 @@ public abstract class AbstractOneDriveResourceAccess {
         return false == isFolder(jItem);
     }
 
-    protected final OneDriveAccess oneDriveAccess;
+    protected final OneDriveOAuthAccess oneDriveAccess;
     protected final Session session;
     protected final FileStorageAccount account;
     protected String rootFolderIdentifier;
@@ -232,7 +249,7 @@ public abstract class AbstractOneDriveResourceAccess {
     /**
      * Initializes a new {@link AbstractOneDriveResourceAccess}.
      */
-    protected AbstractOneDriveResourceAccess(OneDriveAccess oneDriveAccess, FileStorageAccount account, Session session) {
+    protected AbstractOneDriveResourceAccess(OneDriveOAuthAccess oneDriveAccess, FileStorageAccount account, Session session) {
         super();
         this.oneDriveAccess = oneDriveAccess;
         this.account = account;
@@ -255,7 +272,7 @@ public abstract class AbstractOneDriveResourceAccess {
                 try {
                     int keepOn = 1;
                     while (keepOn > 0) {
-                        DefaultHttpClient httpClient = oneDriveAccess.getHttpClient();
+                        DefaultHttpClient httpClient = oneDriveAccess.<DefaultHttpClient> getClient().client;
                         HttpGet method = new HttpGet(buildUri("/me/skydrive", initiateQueryString()));
                         request = method;
                         // HttpClients.setRequestTimeout(3500, method);
@@ -270,7 +287,7 @@ public abstract class AbstractOneDriveResourceAccess {
                             request = null;
                             keepOn = 2;
 
-                            oneDriveAccess.reinit(session);
+                            oneDriveAccess.initialize();
                         } else {
                             JSONObject jResponse = handleHttpResponse(httpResponse, JSONObject.class);
                             rootFolderId = jResponse.optString("id", null);
@@ -326,6 +343,33 @@ public abstract class AbstractOneDriveResourceAccess {
         }
     }
 
+    /**
+     * Closes the supplied HTTP request & response resources silently.
+     *
+     * @param request The HTTP request to reset
+     * @param response The HTTP response to consume and close
+     */
+    public static void close(HttpRequestBase request, HttpResponse response) {
+        if (null != response) {
+            HttpEntity entity = response.getEntity();
+            if (null != entity) {
+                try {
+                    EntityUtils.consume(entity);
+                } catch (Exception e) {
+                    // Ignore
+                }
+            }
+        }
+
+        if (null != request) {
+            try {
+                request.reset();
+            } catch (Exception e) {
+                // Ignore
+            }
+        }
+    }
+
     /**
      * Turns specified JSON value into an appropriate HTTP entity.
      *
@@ -353,7 +397,7 @@ public abstract class AbstractOneDriveResourceAccess {
      */
     protected List<NameValuePair> initiateQueryString() {
         List<NameValuePair> qparams = new LinkedList<NameValuePair>();
-        qparams.add(new BasicNameValuePair("access_token", oneDriveAccess.getAccessToken()));
+        qparams.add(new BasicNameValuePair("access_token", oneDriveAccess.getOAuthAccount().getToken()));
         return qparams;
     }
 
@@ -418,8 +462,6 @@ public abstract class AbstractOneDriveResourceAccess {
         return parseIntoObject(httpResponse.getEntity().getContent(), clazz);
     }
 
-
-
     /**
      * Performs given closure.
      *
@@ -429,7 +471,7 @@ public abstract class AbstractOneDriveResourceAccess {
      * @throws OXException If performing closure fails
      */
     protected <R> R perform(OneDriveClosure<R> closure) throws OXException {
-        return closure.perform(this, oneDriveAccess.getHttpClient(), session);
+        return closure.perform(this, oneDriveAccess.<DefaultHttpClient> getClient().client, session);
     }
 
     /**
@@ -441,7 +483,7 @@ public abstract class AbstractOneDriveResourceAccess {
      */
     protected void handleAuthError(HttpResponseException e, Session session) throws OXException {
         try {
-            oneDriveAccess.reinit(session);
+            oneDriveAccess.initialize();
         } catch (OXException oxe) {
             Logger logger = org.slf4j.LoggerFactory.getLogger(AbstractOneDriveResourceAccess.class);
             logger.warn("Could not re-initialize Microsoft OneDrive access", oxe);
